""" GeneSet module """
from orangecontrib.bioinformatics.geneset.config import *
from orangecontrib.bioinformatics.geneset.utils import (
    filename, GeneSets, GeneSet, NoGeneSetsException, GeneSetException, filename_parse
)
from orangecontrib.bioinformatics.utils import serverfiles


def list_all(**kwargs):
    """ Returns available gene sets from the server files repository: a list of (hierarchy, organism)
    """
    organism = kwargs.get("organism", None)

    all_available = [filename_parse(f_name) for domain, f_name
                     in serverfiles.ServerFiles().listfiles(DOMAIN) + serverfiles.listfiles(DOMAIN)]
    if organism:
        return [(hier, org) for hier, org in all_available if org == organism]
    else:
        return all_available


def load_gene_sets(hierarchy):
    file_path = serverfiles.localpath_download(DOMAIN, filename(*hierarchy))
    return GeneSets.from_gmt_file_format(file_path)



from PyQt5 import QtCore, QtGui, QtWidgets


class FileUploadHelper:
    def setupUi(self, dialog):
        dialog.setObjectName("Add new file")
        #dialog.resize(358, 126)
        self.verticalLayout = QtWidgets.QVBoxLayout(dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.dialoglabel = QtWidgets.QLabel(dialog)
        self.dialoglabel.setObjectName("dialoglabel")
        self.horizontalLayout_3.addWidget(self.dialoglabel)
        self.verticalLayout.addLayout(self.horizontalLayout_3)


        #self.retranslateUi(dialog)
        QtCore.QMetaObject.connectSlotsByName(dialog)

        # connect the two functions
        #self.pushButton.clicked.connect(self.return_accept)
        #self.pushButton_2.clicked.connect(self.return_cancel)

    '''def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.dialoglabel.setText(_translate("Dialog", "dialoglabel"))
        self.pushButton_2.setText(_translate("Dialog", "Acept"))
        self.pushButton.setText(_translate("Dialog", "Cancel"))'''

    # 2 sample functions
    def return_accept(self):
        print("yes")

    def return_cancel(self):
        print("no")

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(670, 492)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.tableWidget = QtWidgets.QTableWidget(self.centralwidget)
        self.tableWidget.setObjectName("tableWidget")
        self.tableWidget.setColumnCount(3)
        self.tableWidget.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(2, item)
        self.horizontalLayout_2.addWidget(self.tableWidget)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.firstbutton = QtWidgets.QPushButton(self.centralwidget)
        self.firstbutton.setObjectName("firstbutton")
        self.horizontalLayout.addWidget(self.firstbutton)
        self.secondbutton = QtWidgets.QPushButton(self.centralwidget)
        self.secondbutton.setObjectName("secondbutton")
        self.horizontalLayout.addWidget(self.secondbutton)
        self.verticalLayout.addLayout(self.horizontalLayout)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 670, 21))
        self.menubar.setObjectName("menubar")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

        self.firstbutton.clicked.connect(self.open_dialog)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        item = self.tableWidget.horizontalHeaderItem(0)
        item.setText(_translate("MainWindow", "First column"))
        item = self.tableWidget.horizontalHeaderItem(1)
        item.setText(_translate("MainWindow", "Second column"))
        item = self.tableWidget.horizontalHeaderItem(2)
        item.setText(_translate("MainWindow", "Third"))
        self.firstbutton.setText(_translate("MainWindow", "Add"))
        self.secondbutton.setText(_translate("MainWindow", "Delete"))

    def open_dialog(self):
        dialog = QtWidgets.QDialog()
        dialog.ui = FileUploadHelper()
        dialog.ui.setupUi(dialog)
        dialog.exec_()
        dialog.show()


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    MainWindow = QtWidgets.QMainWindow()
    ui = Ui_MainWindow()
    ui.setupUi(MainWindow)
    MainWindow.show()
    sys.exit(app.exec_())


""" GeneSets utility functions """
from typing import List, Tuple
from orangecontrib.bioinformatics.geneset.config import GENE_SET_ATTRIBUTES
from orangecontrib.bioinformatics.utils import ensure_type


def filename(hierarchy, organism):  # type: (Tuple[str, str], str) -> str
    """ Obtain a filename for given hierarchy and organism.

    Args:
        hierarchy: GeneSet hierarchy, example: ('GO', 'biological_process')
        organism: Taxonomy ID

    Returns:
        Filename for given hierarchy and organism
    """
    return '-'.join(hierarchy + (organism if organism is not None else '',)) + '.gmt'


def filename_parse(fn):  # type: (str) -> (Tuple[Tuple[str, str], str])
    """ Returns a hierarchy and the organism from the gene set filename format.

    Args:
        fn: GeneSets file name (.gmt)

    Returns:
        A hierarchy and taxonomy id for given filename
    """

    file_name = fn[:-4]  # removes .gmt extension
    parts = file_name.split('-')
    hierarchy = tuple(parts[:-1])
    org = parts[-1] if parts[-1] != '' else None
    return hierarchy, org


class GeneSet:
    __slots__ = GENE_SET_ATTRIBUTES

    def __init__(self, gs_id=None, hierarchy=None, organism=None, name=None, genes=None, description=None, link=None):
        """ Object representing a single set of genes

        Args:
            gs_id: Short gene set ID.
            hierarchy: Hierarchy should be formated as a tuple, for example ``("GO", "biological_process")``
            organism: Organism as a NCBI taxonomy ID.
            name: Gene set name.
            genes: A set of genes. Genes are strings.
            description: Gene set description.
            link: Link to further information about this gene set.
        """

        self.gs_id = gs_id
        self.hierarchy = hierarchy
        self.organism = organism
        self.name = name
        self.genes = genes
        self.description = description
        self.link = link

    def __hash__(self):
        return self.gs_id.__hash__() + self.name.__hash__()

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            if self.__slots__ == other.__slots__:
                return all(getattr(self, attr) == getattr(other, attr) for attr in self.__slots__)

        return False

    def gmt_description(self):
        """ Represent GeneSet as line in GMT file format

        Returns:
             Comma-separated GeneSet attributes.
        """

        # this is just so that we preserve order from GENE_SET_ATTRIBUTES for easy access
        empty_field = '_'
        hierarchy = '-'.join((hier for hier in self.hierarchy)) if self.hierarchy else empty_field

        return ','.join((self.gs_id,
                         hierarchy,
                         self.organism,
                         self.name if self.name else empty_field,
                         empty_field,
                         self.description if self.description else empty_field,
                         self.link if self.link else empty_field))


class GeneSets(set):
    """ A collection of gene sets: contains :class:`GeneSet` objects.
    """
    
    def __init__(self, sets=None):
        # type: (List[GeneSet]) -> None
        super().__init__()

        if sets:
            self.update(ensure_type(sets, list))

    def update(self, sets):
        # type: (List[GeneSet]) -> None

        for g_set in sets:
            self.add(ensure_type(g_set, GeneSet))

    def common_org(self):
        """ Return a common organism. """
        if len(self) == 0:
            raise GeneSetException("Empty gene sets.")

        organisms = set(g_set.organism for g_set in self)
        if not len(organisms) == 1:
            raise GeneSetException("multiple organisms in a set " + str(organisms))
        else:
            for org in organisms:
                return org

    def hierarchies(self):
        """ Return all hierarchies. """
        if len(self) == 0:
            raise GeneSetException("Empty gene sets.")

        return set(g_set.hierarchy for g_set in self)

    def common_hierarchy(self):
        """ Return a common hierarchy. """
        hierarchies = self.hierarchies()

        if not len(hierarchies) == 1:
            raise GeneSetException('multiple hierarchies in a set', str(hierarchies))
        else:
            for org in hierarchies:
                return org

    def split_by_hierarchy(self):
        """ Split gene sets by hierarchies. Return a list of :class:`GeneSets` objects. """
        split_by_hier = {hier: GeneSets() for hier in self.hierarchies()}
        [split_by_hier[gs.hierarchy].update([gs]) for gs in self]

        return list(split_by_hier.values())

    def to_gmt_file_format(self, file_path):  # type: (str) -> None
        """ The GMT file format is a tab delimited file format that describes gene sets.

        In the GMT format, each row represents a gene set.
        Columns: gs_id    gmt_description    Gene    Gene    Gene    ...
        gmt_description: 'gs_id','hierarchy','organism','name','genes','description','link'

        Args:
            file_path: Path to where file will be created


        Returns:
            None

        """

        with open(file_path, 'a') as gmt_file:
            for gene_set in self:
                genes = map(str, gene_set.genes)
                line = '\t'.join((gene_set.gs_id, gene_set.gmt_description(), *genes))
                gmt_file.write(line + '\n')

    @staticmethod
    def from_gmt_file_format(file_path):  # type: (str) -> GeneSets
        """ Load GeneSets object from GMT file.

        Args:
            file_path: GeneSets file path

        Returns:
            GeneSets object

        """
        index = {label: index for index, label in enumerate(GENE_SET_ATTRIBUTES)}

        with open(file_path, 'r') as gmt_file:
            gene_sets = []

            for line in gmt_file:
                columns = [column.strip() for column in line.split('\t')]
                gs_info = columns[1].split(',')
                hierarchy = tuple(gs_info[index['hierarchy']].split('-'))
                genes = set([int(gene) for gene in columns[2:]])

                gene_set = GeneSet(gs_id=columns[0],
                                   genes=genes,
                                   hierarchy=hierarchy,
                                   name=gs_info[index['name']],
                                   organism=gs_info[index['organism']],
                                   description=gs_info[index['description']],
                                   link=gs_info[index['link']])

                gene_sets.append(gene_set)

            return GeneSets(gene_sets)


class NoGeneSetsException(Exception):
    """ Raised when provided taxonomy is not in orangecontrib.bio.ncbi.taxonomy.common_taxids """


class GeneSetException(Exception):
    pass


if __name__ == '__main__':
    pass



import pandas as pd
import requests

from collections import defaultdict
from orangecontrib.bioinformatics.go import Ontology, Annotations

from orangecontrib.bioinformatics.geneset import GeneSets, GeneSet, GO_TERM_LINK


def jaccard_index(set1, set2):
    return round((len(set1.intersection(set2)) / len(set1.union(set2))) * 100, 2)


def go():
    df = pd.ExcelFile('/Users/jakakokosar/OneDrive/Documents/Orange/aplisiya/annotations.xlsx')
    data = df.parse("Annotated")
    map_go_to_genes = defaultdict(list)

    for index, row in data.iterrows():
        gene_id = row['GeneID']
        set1 = set()
        set2 = set()

        if isinstance(row['go_ids_left'], str):
            set1 = set(row['go_ids_left'].strip().split(';')[:-1])

        if isinstance(row['go_ids_right'], str):
            set2 = set(row['go_ids_right'].strip().split(';')[:-1])

        go_ids = set1.union(set2)

        for go_id in go_ids:
            map_go_to_genes[go_id].append(gene_id)

    ontology = Ontology()
    category = {'biological_process': 'Process',
                'molecular_function': 'Function',
                'cellular_component': 'Component'}

    lines = []
    for go, genes in map_go_to_genes.items():
        go_id = 'GO:' + go

        if 'is_obsolete' not in ontology[go_id].values:
            for gene in set(genes):
                lines.append(
                    ['6500', gene, go_id, 'ND', '-', ontology[go_id].values['name'],
                     '-', category[ontology[go_id].values['namespace']]]
                )

    with open('/Users/jakakokosar/OneDrive/Documents/Orange/gene_association.6500', 'w') as file:
        file.write('#tax_id\tGeneID\tGO_ID\tEvidence\tQualifier\tGO_term\tPubMed\tCategory\n')
        file.writelines('\t'.join(i) + '\n' for i in lines)


def go_gene_sets():
    """ Returns gene sets from GO.
    """

    ontology = Ontology()
    annotations = Annotations('6500', ontology=ontology)

    gene_sets_p = []
    gene_sets_f = []
    gene_sets_c = []
    for termn, term in ontology.terms.items():
        genes = annotations.get_genes_by_go_term(termn)
        hier = ('GO', term.namespace)
        if len(genes) > 0:

            gs = GeneSet(gs_id=termn, name=term.name, genes=genes, hierarchy=hier,
                         organism='6500', link=GO_TERM_LINK.format(termn))

            if term.namespace == 'biological_process':
                gene_sets_p.append(gs)
            if term.namespace == 'molecular_function':
                gene_sets_f.append(gs)
            if term.namespace == 'cellular_component':
                gene_sets_c.append(gs)

    bio_p = GeneSets(gene_sets_p)
    mol_f = GeneSets(gene_sets_f)
    cel_c = GeneSets(gene_sets_c)

    bio_p.to_gmt_file_format('/Users/jakakokosar/OneDrive/Documents/Orange/aplisiya/biological_process.gmt')
    mol_f.to_gmt_file_format('/Users/jakakokosar/OneDrive/Documents/Orange/aplisiya/molecular_function.gmt')
    cel_c.to_gmt_file_format('/Users/jakakokosar/OneDrive/Documents/Orange/aplisiya/cellular_component.gmt')


def kegg():
    df = pd.ExcelFile('/Users/jakakokosar/OneDrive/Documents/Orange/aplisiya/annotations.xlsx')
    data = df.parse("Annotated")
    ko_map = {}
    map_kegg_to_genes = defaultdict(list)

    for index, row in data.iterrows():
        gene_id = row['GeneID']
        set1 = set()
        set2 = set()

        if isinstance(row['kegg_ids_left'], str):
            set1 = set(row['kegg_ids_left'].strip().split(';')[:-1])

        if isinstance(row['kegg_ids_right'], str):
            set2 = set(row['kegg_ids_right'].strip().split(';')[:-1])

        kegg_ids = set1.union(set2)

        for go_id in kegg_ids:
            map_kegg_to_genes[go_id].append(gene_id)

    with requests.get("http://rest.kegg.jp/list/ko") as data:
        lines = data.text.split('\n')
        for line in lines:
            kegg_info = line.split('\t')
            try:
                ko_map[kegg_info[0]] = kegg_info[1]
            except IndexError:
                pass
                print(line, kegg_info)
                #raise

    genesets = []

    for kegg_id, genes in map_kegg_to_genes.items():
        kegg_id = 'ko:' + kegg_id

        if kegg_id in ko_map:
            kegg_name, kegg_description = ko_map[kegg_id].split(';', 1)

            gs = GeneSet(gs_id=kegg_id,
                         name=kegg_name,
                         description=kegg_description,
                         genes=genes,
                         hierarchy=('KEGG', 'orthologs'),
                         organism='6500',
                         link='http://www.kegg.jp/entry/{}'.format(kegg_id))

            genesets.append(gs)

    g_sets = GeneSets(genesets)
    g_sets.to_gmt_file_format('/Users/jakakokosar/OneDrive/Documents/Orange/aplisiya/kegg_genesets.gmt')


if __name__ == '__main__':
    #go()
    go_gene_sets()
    #kegg()


#data.to_csv('~/Documents/go_ids_analysis.tab', sep='\t', encoding='utf-8')



gos = ['0001889','0001892','0003690','0006338','0019904','0030154','0030225','0042267',
       '0000122','0001892','0003700','0005515','0006366','0008203','0030099','0035189','0046982',
       '0046983','0005667','0006953','0016071','0030324','0042267','0042803','0043565','0045669',
       '0048469','0000122','0008134','0016071','0019048','0019904','0045600','0045739','0048839','0050872','0050873']



""" Gene Info Widget Display gene summary information from NCBI Gene database. """
import sys
import math
import Orange
import numpy as np

from itertools import chain
from collections import defaultdict, OrderedDict
from functools import partial, lru_cache

from AnyQt.QtWidgets import QTreeView
from AnyQt.QtGui import QFont, QColor
from AnyQt.QtCore import (
    Qt, QSize, QThread, QAbstractItemModel, QSortFilterProxyModel,
    QModelIndex, QItemSelection, QItemSelectionModel, Slot
)

from Orange.widgets import widget, gui, settings
from Orange.widgets.utils.concurrent import ThreadExecutor, Task, methodinvoke
from Orange.widgets.utils.signals import Output, Input

from orangecontrib.bioinformatics.ncbi import gene, taxonomy
from orangecontrib.bioinformatics.utils import serverfiles
from orangecontrib.bioinformatics.widgets.utils.data import TAX_ID, GENE_AS_ATTRIBUTE_NAME
from orangecontrib.bioinformatics.widgets.utils.data import append_columns


def ensure_downloaded(domain, filename, advance=None):
    serverfiles.localpath_download(domain, filename, callback=advance)


class TreeModel(QAbstractItemModel):

    def __init__(self, data, header, parent):
        QAbstractItemModel.__init__(self, parent)
        self._data = data
        self._dataDict = {}
        self._header = header
        self._roleData = {Qt.DisplayRole: self._data}
        self._roleData = partial(
            defaultdict,
            partial(defaultdict,
                    partial(defaultdict, lambda: None)))(self._roleData)

        self.setColumnLinks()

    def setColumnLinks(self):
        font = QFont()
        font.setUnderline(True)

        for i, link in enumerate(self._data):
            self._roleData[gui.LinkRole][i][0] = gene.NCBI_DETAIL_LINK.format(self._roleData[Qt.DisplayRole][i][0])
            self._roleData[Qt.FontRole][i][0] = font
            self._roleData[Qt.ForegroundRole][i][0] = QColor(Qt.blue)

            self._roleData[gui.LinkRole][i][1] = gene.ENSEMBL_DETAIL_LINK.format(self._roleData[Qt.DisplayRole][i][1])
            self._roleData[Qt.FontRole][i][1] = font
            self._roleData[Qt.ForegroundRole][i][1] = QColor(Qt.blue)

    def setRoleData(self, role, row, col, data):
        self._roleData[role][row][col] = data

    def data(self, index, role=Qt.DisplayRole):
        row, col = index.row(), index.column()
        return self._roleData[role][row][col]

    def index(self, row, col, parent=QModelIndex()):
        return self.createIndex(row, col, 0)

    def parent(self, index):
        return QModelIndex()

    def rowCount(self, index=QModelIndex()):
        if index.isValid():
            return 0
        else:
            return len(self._data)

    def columnCount(self, index=QModelIndex()):
        return len(self._header)

    def headerData(self, section, orientation, role=Qt.DisplayRole):
        if role == Qt.DisplayRole:
            return self._header[section]
        return None


@lru_cache(maxsize=2)
def get_ncbi_info(taxid):
    return gene.GeneInfo(taxid)


HEADER_SCHEMA = {label: index for index, label in enumerate(gene.GENE_INFO_HEADER_LABELS)}


def ncbi_info(taxid, genes, advance=None):
    ensure_downloaded(gene.DOMAIN, gene.FILENAME, advance)
    info = get_ncbi_info(taxid)

    gene_matcher = gene.GeneMatcher(str(taxid))
    gene_matcher.genes = genes
    gene_matcher.run_matcher()

    map_input_to_ensembl = OrderedDict({input_name: '' for input_name in genes})
    ret = []

    for gene_obj in gene_matcher.get_known_genes():
        if gene_obj.ncbi_id:
            gi = info.get_gene_by_id(gene_obj.ncbi_id)

            ensemble = ''
            if 'Ensembl' in gi.db_refs:
                ensemble = gi.db_refs['Ensembl']
                map_input_to_ensembl[gene_obj.input_name] = ensemble

            ret.append((gene_obj.input_name,
                        [str(gi.gene_id),
                         str(ensemble),
                         gi.symbol + " (%s)" % gene_obj.input_name if gene_obj.input_name != gi.symbol else gi.symbol,
                         gi.locus_tag or "",
                         gi.chromosome or "",
                         gi.description or "",
                         gi.synonyms,
                         gi.symbol_from_nomenclature_authority or ""])
                       )
        else:
            ret.append(None)

    return map_input_to_ensembl, ret


class OWGeneInfo(widget.OWWidget):
    name = "Gene Info"
    description = "Displays gene information from NCBI and other sources."
    icon = "../widgets/icons/OWGeneInfo.svg"
    priority = 5

    class Inputs:
        data = Input("Data", Orange.data.Table)

    class Outputs:
        selected_genes = Output("Selected Genes", Orange.data.Table)
        data = Output("Data", Orange.data.Table)

    settingsHandler = settings.DomainContextHandler()

    organism_index = settings.ContextSetting(0)
    taxid = settings.ContextSetting("9606")

    gene_attr = settings.ContextSetting(0)

    auto_commit = settings.Setting(False)
    search_string = settings.Setting("")

    useAttr = settings.ContextSetting(False)
    useAltSource = settings.ContextSetting(False)

    def __init__(self, parent=None, ):
        super().__init__(self, parent)

        self.selectionChangedFlag = False

        self.__initialized = False
        self.initfuture = None
        self.itemsfuture = None

        self.map_input_to_ensembl = None
        self.infoLabel = gui.widgetLabel(
            gui.widgetBox(self.controlArea, "Info", addSpace=True),
            "Initializing\n"
        )

        self.organisms = None
        self.organismBox = gui.widgetBox(
            self.controlArea, "Organism", addSpace=True)

        self.organismComboBox = gui.comboBox(
            self.organismBox, self, "organism_index",
            callback=self._onSelectedOrganismChanged)

        box = gui.widgetBox(self.controlArea, "Gene names", addSpace=True)
        self.geneAttrComboBox = gui.comboBox(
            box, self, "gene_attr",
            "Gene attribute", callback=self.updateInfoItems
        )
        self.geneAttrComboBox.setEnabled(not self.useAttr)

        self.geneAttrCheckbox = gui.checkBox(box, self, "useAttr", "Use column names",
                                             callback=self.updateInfoItems)
        self.geneAttrCheckbox.toggled[bool].connect(self.geneAttrComboBox.setDisabled)

        gui.auto_commit(self.controlArea, self, "auto_commit", "Commit")

        gui.rubber(self.controlArea)

        gui.lineEdit(self.mainArea, self, "search_string", "Filter",
                     callbackOnType=True, callback=self.searchUpdate)

        self.treeWidget = QTreeView(self.mainArea)

        self.treeWidget.setAlternatingRowColors(True)
        self.treeWidget.setSortingEnabled(True)
        self.treeWidget.setSelectionMode(QTreeView.ExtendedSelection)
        self.treeWidget.setUniformRowHeights(True)
        self.treeWidget.setRootIsDecorated(False)

        self.treeWidget.setItemDelegateForColumn(HEADER_SCHEMA['NCBI ID'], gui.LinkStyledItemDelegate(self.treeWidget))
        self.treeWidget.setItemDelegateForColumn(HEADER_SCHEMA['Ensembl ID'], gui.LinkStyledItemDelegate(self.treeWidget))

        self.treeWidget.viewport().setMouseTracking(True)
        self.mainArea.layout().addWidget(self.treeWidget)

        box = gui.widgetBox(self.mainArea, "", orientation="horizontal")
        gui.button(box, self, "Select Filtered", callback=self.selectFiltered)
        gui.button(box, self, "Clear Selection",
                   callback=self.treeWidget.clearSelection)

        self.geneinfo = []
        self.cells = []
        self.row2geneinfo = {}
        self.data = None

        # : (# input genes, # matches genes)
        self.matchedInfo = 0, 0

        self.setBlocking(True)
        self.executor = ThreadExecutor(self)

        self.progressBarInit()

        task = Task(
            function=partial(
                taxonomy.ensure_downloaded,
                callback=methodinvoke(self, "advance", ())
            )
        )

        task.resultReady.connect(self.initialize)
        task.exceptionReady.connect(self._onInitializeError)

        self.initfuture = self.executor.submit(task)

    def sizeHint(self):
        return QSize(1024, 720)

    @Slot()
    def advance(self):
        assert self.thread() is QThread.currentThread()
        self.progressBarSet(self.progressBarValue + 1,
                            processEvents=None)

    def _get_available_organisms(self):
        available_organism = sorted([(tax_id, taxonomy.name(tax_id)) for tax_id in taxonomy.common_taxids()],
                                    key=lambda x: x[1])

        self.organisms = [tax_id[0] for tax_id in available_organism]

        self.organismComboBox.addItems([tax_id[1] for tax_id in available_organism])

    def initialize(self):
        if self.__initialized:
            # Already initialized
            return
        self.__initialized = True

        self._get_available_organisms()
        self.organism_index = self.organisms.index(taxonomy.DEFAULT_ORGANISM)
        self.taxid = self.organisms[self.organism_index]

        self.infoLabel.setText("No data on input\n")
        self.initfuture = None

        self.setBlocking(False)
        self.progressBarFinished(processEvents=None)

    def _onInitializeError(self, exc):
        sys.excepthook(type(exc), exc, None)
        self.error(0, "Could not download the necessary files.")

    def _onSelectedOrganismChanged(self):
        assert 0 <= self.organism_index <= len(self.organisms)
        self.taxid = self.organisms[self.organism_index]

        if self.data is not None:
            self.updateInfoItems()

    @Inputs.data
    def setData(self, data=None):
        if not self.__initialized:
            self.initfuture.result()
            self.initialize()

        if self.itemsfuture is not None:
            raise Exception("Already processing")

        self.data = data

        if data is not None:
            self.geneAttrComboBox.clear()
            self.attributes = [attr for attr in data.domain.variables + data.domain.metas
                               if isinstance(attr, (Orange.data.StringVariable, Orange.data.DiscreteVariable))]

            for var in self.attributes:
                self.geneAttrComboBox.addItem(*gui.attributeItem(var))

            self.taxid = str(self.data.attributes.get(TAX_ID, ''))
            self.useAttr = self.data.attributes.get(GENE_AS_ATTRIBUTE_NAME, self.useAttr)

            self.gene_attr = min(self.gene_attr, len(self.attributes) - 1)

            if self.taxid in self.organisms:
                self.organism_index = self.organisms.index(self.taxid)

            self.updateInfoItems()
        else:
            self.clear()

    def updateInfoItems(self):
        self.warning(0)
        if self.data is None:
            return

        if self.useAttr:
            genes = [attr.name for attr in self.data.domain.attributes]
        elif self.attributes:
            attr = self.attributes[self.gene_attr]
            genes = [str(ex[attr]) for ex in self.data
                     if not math.isnan(ex[attr])]
        else:
            genes = []
        if not genes:
            self.warning(0, "Could not extract genes from input dataset.")

        self.warning(1)
        org = self.organisms[min(self.organism_index, len(self.organisms) - 1)]
        source_name, info_getter = ("NCBI Info", ncbi_info)

        self.error(0)

        self.progressBarInit()
        self.setBlocking(True)
        self.setEnabled(False)
        self.infoLabel.setText("Retrieving info records.\n")

        self.genes = genes

        task = Task(
            function=partial(
                info_getter, org, genes,
                advance=methodinvoke(self, "advance", ()))
        )
        self.itemsfuture = self.executor.submit(task)
        task.finished.connect(self._onItemsCompleted)

    def _onItemsCompleted(self):
        self.setBlocking(False)
        self.progressBarFinished()
        self.setEnabled(True)

        try:
            self.map_input_to_ensembl, geneinfo = self.itemsfuture.result()
        finally:
            self.itemsfuture = None

        self.geneinfo = geneinfo
        self.cells = cells = []
        self.row2geneinfo = {}

        for i, (input_name, gi) in enumerate(geneinfo):
            if gi:
                row = []
                for item in gi:
                    row.append(item)

                # parse synonyms
                row[HEADER_SCHEMA['Synonyms']] = ','.join(row[HEADER_SCHEMA['Synonyms']])
                cells.append(row)
                self.row2geneinfo[len(cells) - 1] = i

        model = TreeModel(cells, list(HEADER_SCHEMA.keys()), None)

        proxyModel = QSortFilterProxyModel(self)
        proxyModel.setSourceModel(model)
        self.treeWidget.setModel(proxyModel)
        self.treeWidget.selectionModel().selectionChanged.connect(self.commit)

        for i in range(len(HEADER_SCHEMA)):
            self.treeWidget.resizeColumnToContents(i)
            self.treeWidget.setColumnWidth(
                i, min(self.treeWidget.columnWidth(i), 200)
            )

        self.infoLabel.setText("%i genes\n%i matched NCBI's IDs" %
                               (len(self.genes), len(cells)))
        self.matchedInfo = len(self.genes), len(cells)

        if self.useAttr:
            new_data = self.data.from_table(self.data.domain, self.data)

            for gene_var in new_data.domain.attributes:
                 gene_var.attributes['Ensembl ID'] = str(self.map_input_to_ensembl[gene_var.name])

            self.Outputs.data.send(new_data)

        elif self.attributes:
            ensembl_ids = []
            for gene_name in self.data.get_column_view(self.attributes[self.gene_attr])[0]:
                if gene_name and gene_name in self.map_input_to_ensembl:
                    ensembl_ids.append(self.map_input_to_ensembl[gene_name])
                else:
                    ensembl_ids.append('')

            data_with_ensembl = append_columns(self.data,
                                               metas=[(Orange.data.StringVariable('Ensembl ID'), ensembl_ids)])
            self.Outputs.data.send(data_with_ensembl)

    def clear(self):
        self.infoLabel.setText("No data on input\n")
        self.treeWidget.setModel(
            TreeModel([], ["NCBI ID", "Symbol", "Locus Tag",
                           "Chromosome", "Description", "Synonyms",
                           "Nomenclature"], self.treeWidget))

        self.geneAttrComboBox.clear()
        self.Outputs.selected_genes.send(None)

    def commit(self):
        if self.data is None:
            self.Outputs.selected_genes.send(None)
            self.Outputs.data.send(None)
            return

        model = self.treeWidget.model()
        selection = self.treeWidget.selectionModel().selection()
        selection = model.mapSelectionToSource(selection)
        selectedRows = list(
            chain(*(range(r.top(), r.bottom() + 1) for r in selection))
        )
        model = model.sourceModel()

        selectedGeneids = [self.row2geneinfo[row] for row in selectedRows]
        selectedIds = [self.geneinfo[i][0] for i in selectedGeneids]
        selectedIds = set(selectedIds)
        gene2row = dict((self.geneinfo[self.row2geneinfo[row]][0], row)
                        for row in selectedRows)

        isselected = selectedIds.__contains__

        if selectedIds:

            if self.useAttr:
                attrs = [attr for attr in self.data.domain.attributes if isselected(attr.name)]
                domain = Orange.data.Domain(attrs, self.data.domain.class_vars, self.data.domain.metas)
                newdata = self.data.from_table(domain, self.data)

                self.Outputs.selected_genes.send(newdata)

            elif self.attributes:
                attr = self.attributes[self.gene_attr]
                gene_col = [attr.str_val(v) for v in self.data.get_column_view(attr)[0]]
                gene_col = [(i, name) for i, name in enumerate(gene_col) if isselected(name)]
                indices = [i for i, _ in gene_col]

                # SELECTED GENES OUTPUT
                selected_genes_metas = [Orange.data.StringVariable(name) for name in gene.GENE_INFO_HEADER_LABELS]
                selected_genes_domain = Orange.data.Domain(
                    self.data.domain.attributes, self.data.domain.class_vars,
                    self.data.domain.metas + tuple(selected_genes_metas))

                selected_genes_data = self.data.from_table(selected_genes_domain, self.data)[indices]

                model_rows = [gene2row[gene_name] for _, gene_name in gene_col]
                for col, meta in zip(range(model.columnCount()), selected_genes_metas):
                    col_data = [str(model.index(row, col).data(Qt.DisplayRole)) for row in model_rows]
                    col_data = np.array(col_data, dtype=object, ndmin=2).T
                    selected_genes_data[:, meta] = col_data

                if not len(selected_genes_data):
                    selected_genes_data = None

                self.Outputs.selected_genes.send(selected_genes_data)
        else:
            self.Outputs.selected_genes.send(None)

    def rowFiltered(self, row):
        searchStrings = self.search_string.lower().split()
        row = " ".join(self.cells[row]).lower()
        return not all([s in row for s in searchStrings])

    def searchUpdate(self):
        if not self.data:
            return
        searchStrings = self.search_string.lower().split()
        index = self.treeWidget.model().sourceModel().index
        mapFromSource = self.treeWidget.model().mapFromSource
        for i, row in enumerate(self.cells):
            row = " ".join(row).lower()
            self.treeWidget.setRowHidden(
                mapFromSource(index(i, 0)).row(),
                QModelIndex(),
                not all([s in row for s in searchStrings]))

    def selectFiltered(self):
        if not self.data:
            return
        itemSelection = QItemSelection()

        index = self.treeWidget.model().sourceModel().index
        mapFromSource = self.treeWidget.model().mapFromSource
        for i, row in enumerate(self.cells):
            if not self.rowFiltered(i):
                itemSelection.select(mapFromSource(index(i, 0)),
                                     mapFromSource(index(i, 0)))
        self.treeWidget.selectionModel().select(
            itemSelection,
            QItemSelectionModel.Select | QItemSelectionModel.Rows)

    def onAltSourceChange(self):
        self.updateInfoItems()

    def onDeleteWidget(self):
        # try to cancel pending tasks
        if self.initfuture:
            self.initfuture.cancel()
        if self.itemsfuture:
            self.itemsfuture.cancel()

        self.executor.shutdown(wait=False)
        super().onDeleteWidget()


def reportItemView(view):
    model = view.model()
    return reportItemModel(view, model)


def reportItemModel(view, model, index=QModelIndex()):
    if not index.isValid() or model.hasChildren(index):
        columnCount, rowCount = model.columnCount(index), model.rowCount(index)
        if not index.isValid():
            text = ('<table>\n<tr>' +
                    ''.join('<th>%s</th>' %
                            model.headerData(i, Qt.Horizontal, Qt.DisplayRole)
                            for i in range(columnCount)) +
                    '</tr>\n')
        else:
            pass
        text += ''.join('<tr>' +
                        ''.join('<td>' + reportItemModel(view, model, model.index(row, column, index)) +
                                '</td>' for column in range(columnCount)) +
                        '</tr>\n'
                        for row in range(rowCount)
                        if not view.isRowHidden(row, index))
        text += '</table>'
        return text
    else:
        variant = model.data(index, Qt.DisplayRole)
        return str(variant)


def test_main(argv=sys.argv):
    from AnyQt.QtWidgets import QApplication
    app = QApplication(argv)

    if len(argv) > 1:
        filename = argv[1]
    else:
        filename = "brown-selected"

    data = Orange.data.Table(filename)
    w = OWGeneInfo()
    w.setData(data)
    w.show()
    w.raise_()
    r = app.exec_()
    w.saveSettings()
    return r


if __name__ == "__main__":
    sys.exit(test_main())



from PyQt5 import QtWidgets, QtCore

class Delegate(QtWidgets.QItemDelegate):
    def __init__(self, owner, choices):
        super().__init__(owner)
        self.items = choices
    def createEditor(self, parent, option, index):
        self.editor = QtWidgets.QComboBox(parent)
        self.editor.addItems(self.items)
        return self.editor
    def paint(self, painter, option, index):
        value = index.data(QtCore.Qt.DisplayRole)
        style = QtWidgets.QApplication.style()
        opt = QtWidgets.QStyleOptionComboBox()
        opt.text = str(value)
        opt.rect = option.rect
        style.drawComplexControl(QtWidgets.QStyle.CC_ComboBox, opt, painter)
        QtWidgets.QItemDelegate.paint(self, painter, option, index)
    def setEditorData(self, editor, index):
        value = index.data(QtCore.Qt.DisplayRole)
        num = self.items.index(value)
        editor.setCurrentIndex(num)
    def setModelData(self, editor, model, index):
        value = editor.currentText()
        model.setData(index, QtCore.Qt.DisplayRole, QtCore.QVariant(value))
    def updateEditorGeometry(self, editor, option, index):
        editor.setGeometry(option.rect)

class Model(QtCore.QAbstractTableModel):
    def __init__(self, table):
        super().__init__()
        self.table = table
    def rowCount(self, parent):
        return len(self.table)
    def columnCount(self, parent):
        return len(self.table[0])
    def flags(self, index):
        return QtCore.Qt.ItemIsEditable | QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable
    def data(self, index, role):
        if role == QtCore.Qt.DisplayRole:
            return self.table[index.row()][index.column()]
    def setData(self, index, value, role):
        if role == QtCore.Qt.EditRole:
            self.table[index.row()][index.column()] = value
        return True

class Main(QtWidgets.QMainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)
        # set combo box choices:
        choices = ['apple', 'orange', 'banana']
        # create table data:
        table   = []
        table.append(['A', choices[0]])
        table.append(['B', choices[0]])
        table.append(['C', choices[0]])
        table.append(['D', choices[0]])
        # create table view:
        self.model     = Model(table)
        self.tableView = QtWidgets.QTableView()
        self.tableView.setModel(self.model)
        self.tableView.setItemDelegateForColumn(1, Delegate(self,choices))
        # make combo boxes editable with a single-click:
        for row in range( len(table) ):
            self.tableView.openPersistentEditor(self.model.index(row, 1))
        # initialize
        self.setCentralWidget(self.tableView)
        self.setWindowTitle('Delegate Test')
        self.show()

if __name__ == '__main__':
    import sys
    app = QtWidgets.QApplication(sys.argv)
    main = Main()
    app.exec_()